/* Algoritmul lui Kruskal
 * Det. Arborele Partial de Cost Minim
 * Complezitate: O(m * log2(m))
 */ 
#include <fstream>
#include <algorithm>
#include <vector>
using namespace std;

ifstream fin("kruskal.in");
ofstream fout("kruskal.out");

class DS {
	using VI = vector<int>;
public:
	DS(int n) : 
		n {n}, t {VI(n + 1)}, h {VI(n + 1)}
	{
		for (int i = 1; i <= n; ++i)
			t[i] = i;
	}
	
	int Find(int x)
	{
		if (x == t[x])	
			return x;
		return t[x] = Find(t[x]);
	}
	
	void Union(int r1, int r2)
	{
		if (h[r1] > h[r2])
			t[r2] = r1;
		else
		{
			t[r1] = r2;
			
			if (h[r1] == h[r2])
				h[r2]++;
		}
	}
	
private:
	int n;
	VI t, h;
};

struct Edge {
	Edge(int a, int b, int w) :
		a {a}, b {b}, w {w}
	{}
	
	bool operator < (const Edge& e) const
	{
		return w < e.w;
	}
	
	int a, b, w;
};

int n, m;            // noduri, muchii
int cmin;            // costul APM
vector<Edge> edges;  // retine graful ca vect de muchii
vector<Edge> apm;

void CitesteGraf();
void Kruskal();
void AfiseazaAPM();

int main()
{
	CitesteGraf();
	
	Kruskal();
	AfiseazaAPM();
}
   
void CitesteGraf()
{
	fin >> n;
	int a, b, w;
	while (fin >> a >> b >> w)
		edges.emplace_back(a, b, w);
}

void Kruskal() // alg greedy
{
	int c1, c2;  // componentele conexe (arborii) din care fac parte nodurile a si b	
	
	DS ds(n);
	
	sort(edges.begin(), edges.end()); // O(m * log(m));
	
	for (const Edge& edge : edges)
	{
		// find - O(1)
		c1 = ds.Find(edge.a); // O(1) - amortizat
		c2 = ds.Find(edge.b);
		if (c1 != c2) // a si b fac parte din arbori diferiti
		{
			cmin += edge.w;
			apm.emplace_back(edge);
			
			ds.Union(c1, c2);  // O(1) 
			if (apm.size() == size_t(n - 1))    // am obtinut APM
				break;
		}
	}
}

void AfiseazaAPM()
{
	fout << cmin << '\n';
	
	for (const Edge& edge : apm)
		fout << edge.a << ' ' << edge.b << ' '
			 << edge.w << '\n';
}


